@@ -18,15 +18,16 @@ import os
 import pwd
 import signal
 import subprocess
+import sys
 import time
 
 from storlets.sbus import SBus
-from storlets.sbus.datagram import FDMetadata, SBusServiceDatagram
-from storlets.sbus.command import SBUS_CMD_HALT, SBUS_CMD_PING
-from storlets.sbus.file_description import SBUS_FD_SERVICE_OUT
-from storlets.agent.common.server import command_handler, CommandSuccess, \
-    CommandFailure, SBusServer
-from storlets.agent.common.utils import get_logger
+from storlets.sbus.client import SBusClient
+from storlets.sbus.client.exceptions import SBusClientException, \
+    SBusClientSendError
+from storlets.agent.common.server import command_handler, EXIT_FAILURE, \
+    CommandSuccess, CommandFailure, SBusServer
+from storlets.agent.common.utils import get_logger, DEFAULT_PY2, DEFAULT_PY3
 
 
 class SDaemonError(Exception):
@@ -69,7 +70,8 @@ class StorletDaemonFactory(SBusServer):
         :returns: (A list of the JVM arguments, A list of environ parameters)
         """
 
-        str_prfx = "/usr/local/lib/storlets"
+        lib_dir = "/usr/local/lib/storlets"
+        java_lib_dir = os.path.join(lib_dir, "java")
 
         jar_deps = ['logback-classic-1.1.2.jar',
                     'logback-core-1.1.2.jar',
@@ -79,9 +81,9 @@ class StorletDaemonFactory(SBusServer):
                     'SCommon.jar',
                     'SDaemon.jar',
                     '']
-        jar_deps = [os.path.join(str_prfx, x) for x in jar_deps]
-        str_dmn_clspth = ':'.join(jar_deps) + ':' + storlet_path
-        str_library_path = str_prfx
+        jar_deps = [os.path.join(java_lib_dir, x) for x in jar_deps]
+        str_dmn_clspth = ':'.join(jar_deps + [storlet_path])
+        str_library_path = ':'.join([lib_dir, java_lib_dir])
 
         str_daemon_main_class = "org.openstack.storlet.daemon.SDaemon"
 
@@ -100,10 +102,19 @@ class StorletDaemonFactory(SBusServer):
         return pargs, env
 
     def get_python_args(self, daemon_language, storlet_path, storlet_name,
-                        pool_size, uds_path, log_level):
+                        pool_size, uds_path, log_level,
+                        daemon_language_version):
+        daemon_language_version = daemon_language_version or 3
+        # TODO(takashi): Drop Py2 support
+        if int(float(daemon_language_version)) == 2:
+            daemon_language_version = DEFAULT_PY2
+        else:
+            daemon_language_version = DEFAULT_PY3
+
+        python_interpreter = '/usr/bin/python%s' % daemon_language_version
         str_daemon_main_file = '/usr/local/libexec/storlets/storlets-daemon'
-        pargs = [str_daemon_main_file, storlet_name, uds_path, log_level,
-                 str(pool_size), self.container_id]
+        pargs = [python_interpreter, str_daemon_main_file, storlet_name,
+                 uds_path, log_level, str(pool_size), self.container_id]
 
         python_path = os.path.join('/home/swift/', storlet_name)
         if os.environ.get('PYTHONPATH'):
@@ -129,7 +140,7 @@ class StorletDaemonFactory(SBusServer):
         # TODO(takashi): We had better use contextmanager
         # TODO(takashi): Where is this closed?
         try:
-            dn = open(os.devnull, 'w')
+            dn = open(os.devnull, 'wb')
             daemon_p = subprocess.Popen(
                 pargs, stdout=dn, stderr=subprocess.PIPE,
                 close_fds=True, shell=False, env=env)
@@ -179,27 +190,23 @@ class StorletDaemonFactory(SBusServer):
         storlet_pipe_name = self.storlet_name_to_pipe_name[storlet_name]
         self.logger.debug('Send PING command to {0} via {1}'.
                           format(storlet_name, storlet_pipe_name))
-        read_fd, write_fd = os.pipe()
-        try:
-            dtg = SBusServiceDatagram(
-                SBUS_CMD_PING,
-                [write_fd],
-                [FDMetadata(SBUS_FD_SERVICE_OUT).to_dict()])
-            for i in range(self.NUM_OF_TRIES_PINGING_STARTING_DAEMON):
-                ret = SBus.send(storlet_pipe_name, dtg)
-                if ret >= 0:
-                    resp = os.read(read_fd, 128)
-                    if resp.startswith('True'):
-                        return True
-                    time.sleep(1)
-            else:
-                return False
-        finally:
-            os.close(read_fd)
-            os.close(write_fd)
+        client = SBusClient(storlet_pipe_name)
+        for i in range(self.NUM_OF_TRIES_PINGING_STARTING_DAEMON):
+            try:
+                resp = client.ping()
+                if resp.status:
+                    return True
+            except SBusClientSendError:
+                pass
+            except SBusClientException:
+                self.logger.exception('Failed to send sbus command')
+                break
+            time.sleep(1)
+        return False
 
     def process_start_daemon(self, daemon_language, storlet_path, storlet_name,
-                             pool_size, uds_path, log_level):
+                             pool_size, uds_path, log_level,
+                             daemon_language_version=None):
         """
         Start storlet daemon process
 
@@ -211,6 +218,8 @@ class StorletDaemonFactory(SBusServer):
                           pool provides
         :param uds_path: Path to pipe daemon is going to listen to
         :param log_level: Logger verbosity level
+        :param daemon_language_version: daemon language version (e.g. py2, py3)
+            only python lang supports this option
 
         :returns: True if it starts a new subprocess
                   False if there already exists a running process
@@ -222,7 +231,7 @@ class StorletDaemonFactory(SBusServer):
         elif daemon_language.lower() == 'python':
             pargs, env = self.get_python_args(
                 daemon_language, storlet_path, storlet_name,
-                pool_size, uds_path, log_level)
+                pool_size, uds_path, log_level, daemon_language_version)
         else:
             raise SDaemonError(
                 'Got unsupported daemon language: %s' % daemon_language)
@@ -401,25 +410,20 @@ class StorletDaemonFactory(SBusServer):
         self.logger.debug('Send HALT command to {0} via {1}'.
                           format(storlet_name, storlet_pipe_name))
 
-        read_fd, write_fd = os.pipe()
+        client = SBusClient(storlet_pipe_name)
         try:
-            dtg = SBusServiceDatagram(
-                SBUS_CMD_HALT,
-                [write_fd],
-                [FDMetadata(SBUS_FD_SERVICE_OUT).to_dict()])
-            rc = SBus.send(storlet_pipe_name, dtg)
-            os.close(write_fd)
-            if rc < 0:
+            resp = client.halt()
+            if not resp.status:
+                self.logger.error('Failed to send sbus command: %s' %
+                                  resp.message)
                 raise SDaemonError(
-                    'Failed to send halt command to the storlet daemon {0}'
-                    .format(storlet_name))
-            resp = os.read(read_fd, 128)
-            if not resp.startswith('True'):
-                raise SDaemonError(
-                    'Failed to send halt command to the storlet daemon {0}'
-                    .format(storlet_name))
-        finally:
-            os.close(read_fd)
+                    'Failed to send halt to {0}'.format(storlet_name))
+
+        except SBusClientException:
+            self.logger.exception('Failed to send sbus command')
+            raise SDaemonError(
+                'Failed to send halt command to the storlet daemon {0}'
+                .format(storlet_name))
 
         try:
             os.waitpid(dmn_pid, 0)
@@ -439,7 +443,9 @@ class StorletDaemonFactory(SBusServer):
             if self.process_start_daemon(
                     params['daemon_language'], params['storlet_path'],
                     storlet_name, params['pool_size'],
-                    params['uds_path'], params['log_level']):
+                    params['uds_path'], params['log_level'],
+                    daemon_language_version=params.get(
+                        'daemon_language_version')):
                 msg = 'OK'
             else:
                 msg = '{0} is already running'.format(storlet_name)
@@ -517,15 +523,22 @@ def main():
     # Initialize logger
     logger = get_logger("daemon-factory", opts.log_level, opts.container_id)
     logger.debug("Daemon factory started")
-    SBus.start_logger("DEBUG", container_id=opts.container_id)
 
-    # Impersonate the swift user
-    pw = pwd.getpwnam('swift')
-    os.setresgid(pw.pw_gid, pw.pw_gid, pw.pw_gid)
-    os.setresuid(pw.pw_uid, pw.pw_uid, pw.pw_uid)
+    try:
+        SBus.start_logger("DEBUG", container_id=opts.container_id)
 
-    # create an instance of daemon_factory
-    factory = StorletDaemonFactory(opts.sbus_path, logger, opts.container_id)
+        # Impersonate the swift user
+        pw = pwd.getpwnam('swift')
+        os.setresgid(pw.pw_gid, pw.pw_gid, pw.pw_gid)
+        os.setresuid(pw.pw_uid, pw.pw_uid, pw.pw_uid)
 
-    # Start the main loop
-    return factory.main_loop()
+        # create an instance of daemon_factory
+        factory = StorletDaemonFactory(opts.sbus_path, logger,
+                                       opts.container_id)
+
+        # Start the main loop
+        sys.exit(factory.main_loop())
+
+    except Exception:
+        logger.eception('Unhandled exception')
+        sys.exit(EXIT_FAILURE)
