@@ -26,11 +26,10 @@ import eventlet
 import json
 from contextlib import contextmanager
 
-from storlets.sbus import SBus
-from storlets.sbus.datagram import FDMetadata, SBusServiceDatagram, \
-    SBusExecuteDatagram
+from storlets.sbus.client import SBusClient
+from storlets.sbus.client.exceptions import SBusClientException
+from storlets.sbus.datagram import SBusFileDescriptor
 from storlets.sbus import file_description as sbus_fd
-from storlets.sbus import command as sbus_cmd
 from storlets.gateway.common.exceptions import StorletRuntimeException, \
     StorletTimeout
 from storlets.gateway.common.logger import StorletLogger
@@ -42,19 +41,6 @@ MAX_METADATA_SIZE = 4096
 eventlet.monkey_patch()
 
 
-@contextmanager
-def _open_pipe():
-    """
-    Context manager for os.pipe
-    """
-    read_fd, write_fd = os.pipe()
-    try:
-        yield (read_fd, write_fd)
-    finally:
-        os.close(read_fd)
-        os.close(write_fd)
-
-
 """---------------------------------------------------------------------------
 Sandbox API
 """
@@ -124,79 +110,77 @@ class RunTimePaths(object):
         :param conf: gateway conf
         """
         self.scope = scope
-        self.factory_pipe_suffix = 'factory_pipe'
-        self.sandbox_pipe_prefix = '/mnt/channels'
-        self.storlet_pipe_suffix = '_storlet_pipe'
+        self.factory_pipe_name = 'factory_pipe'
+        self.sandbox_pipe_dir = '/mnt/channels'
 
-        self.sandbox_storlet_dir_prefix = '/home/swift'
-        self.host_root = conf.get('host_root', '/home/docker_device')
-        self.host_pipe_root = \
+        self.sandbox_storlet_base_dir = '/home/swift'
+        self.host_root_dir = conf.get('host_root', '/home/docker_device')
+        self.host_pipe_root_dir = \
             conf.get('pipes_dir',
-                     os.path.join(self.host_root, 'pipes', 'scopes'))
-        self.host_storlet_root = \
+                     os.path.join(self.host_root_dir, 'pipes', 'scopes'))
+        self.host_storlet_root_dir = \
             conf.get('storlets_dir',
-                     os.path.join(self.host_root, 'storlets', 'scopes'))
-        self.host_log_path_root = \
+                     os.path.join(self.host_root_dir, 'storlets', 'scopes'))
+        self.host_log_root_dir = \
             conf.get('log_dir',
-                     os.path.join(self.host_root, 'logs', 'scopes'))
-        self.host_cache_root = \
+                     os.path.join(self.host_root_dir, 'logs', 'scopes'))
+        self.host_cache_root_dir = \
             conf.get('cache_dir',
-                     os.path.join(self.host_root, 'cache', 'scopes'))
+                     os.path.join(self.host_root_dir, 'cache', 'scopes'))
         self.host_restart_script_dir = \
             conf.get('script_dir',
-                     os.path.join(self.host_root, 'scripts'))
+                     os.path.join(self.host_root_dir, 'scripts'))
 
-        self.host_storlet_python_lib_dir = ('/usr/local/lib/python2.7/'
-                                            'dist-packages/storlets')
-        self.sandbox_storlet_python_lib_dir = ('/usr/local/lib/python2.7/'
-                                               'dist-packages/storlets')
         self.host_storlet_native_lib_dir = '/usr/local/lib/storlets'
         self.sandbox_storlet_native_lib_dir = '/usr/local/lib/storlets'
         self.host_storlet_native_bin_dir = '/usr/local/libexec/storlets'
         self.sandbox_storlet_native_bin_dir = '/usr/local/libexec/storlets'
 
-    def host_pipe_prefix(self):
-        return os.path.join(self.host_pipe_root, self.scope)
+    @property
+    def host_pipe_dir(self):
+        return os.path.join(self.host_pipe_root_dir, self.scope)
 
-    def create_host_pipe_prefix(self):
-        path = self.host_pipe_prefix()
+    def create_host_pipe_dir(self):
+        path = self.host_pipe_dir
         if not os.path.exists(path):
             os.makedirs(path)
         # 0777 should be 0700 when we get user namespaces in Docker
         os.chmod(path, stat.S_IRWXU | stat.S_IRWXG | stat.S_IRWXO)
+        return path
 
+    @property
     def host_factory_pipe(self):
-        return os.path.join(self.host_pipe_prefix(),
-                            self.factory_pipe_suffix)
+        return os.path.join(self.host_pipe_dir, self.factory_pipe_name)
 
-    def host_storlet_pipe(self, storlet_id):
-        return os.path.join(self.host_pipe_prefix(),
-                            storlet_id)
+    def get_host_storlet_pipe(self, storlet_id):
+        return os.path.join(self.host_pipe_dir, storlet_id)
 
-    def sbox_storlet_pipe(self, storlet_id):
-        return os.path.join(self.sandbox_pipe_prefix,
-                            storlet_id)
+    def get_sbox_storlet_pipe(self, storlet_id):
+        return os.path.join(self.sandbox_pipe_dir, storlet_id)
 
-    def sbox_storlet_exec(self, storlet_id):
-        return os.path.join(self.sandbox_storlet_dir_prefix, storlet_id)
+    def get_sbox_storlet_dir(self, storlet_id):
+        return os.path.join(self.sandbox_storlet_base_dir, storlet_id)
 
-    def host_storlet_prefix(self):
-        return os.path.join(self.host_storlet_root, self.scope)
+    @property
+    def host_storlet_base_dir(self):
+        return os.path.join(self.host_storlet_root_dir, self.scope)
 
-    def host_storlet(self, storlet_id):
-        return os.path.join(self.host_storlet_prefix(), storlet_id)
+    def get_host_storlet_dir(self, storlet_id):
+        return os.path.join(self.host_storlet_base_dir, storlet_id)
 
-    def slog_path(self, storlet_id):
-        log_dir = os.path.join(self.host_log_path_root, self.scope, storlet_id)
-        if not os.path.exists(log_dir):
-            os.makedirs(log_dir)
-        return log_dir
+    def get_host_slog_path(self, storlet_id):
+        return os.path.join(
+            self.host_log_root_dir, self.scope, storlet_id,
+            'storlet_invoke.log')
 
-    def get_host_storlet_cache_dir(self):
-        return os.path.join(self.host_cache_root, self.scope, 'storlet')
+    @property
+    def host_storlet_cache_dir(self):
+        return os.path.join(self.host_cache_root_dir, self.scope, 'storlet')
+
+    @property
+    def host_dependency_cache_dir(self):
+        return os.path.join(self.host_cache_root_dir, self.scope, 'dependency')
 
-    def get_host_dependency_cache_dir(self):
-        return os.path.join(self.host_cache_root, self.scope, 'dependency')
 
 """---------------------------------------------------------------------------
 Docker Stateful Container API
@@ -246,20 +230,9 @@ class RunTimeSandbox(object):
         # TODO(change logger's route if possible)
         self.logger = logger
 
-    def _parse_sandbox_factory_answer(self, str_answer):
-        """
-        Parse answer string received from container side
-
-        :param str_answer: answer string
-        :returns: (status, message)
-        """
-        two_tokens = str_answer.split(':', 1)
-        if len(two_tokens) != 2:
-            self.logger.error('Got wrong format about answer over sbus: %s' %
-                              str_answer)
-            raise StorletRuntimeException('Got wrong answer')
-        status = (two_tokens[0] == 'True')
-        return status, two_tokens[1]
+        self.default_docker_image_name = \
+            conf.get('default_docker_image_name',
+                     'ubuntu_18.04_jre11_storlets')
 
     def ping(self):
         """
@@ -269,24 +242,18 @@ class RunTimeSandbox(object):
                    0 when the daemon factory is not responsive
                   -1 when it fails to send command to the process
         """
-        pipe_path = self.paths.host_factory_pipe()
-
-        with _open_pipe() as (read_fd, write_fd):
-            dtg = SBusServiceDatagram(
-                sbus_cmd.SBUS_CMD_PING,
-                [write_fd],
-                [FDMetadata(sbus_fd.SBUS_FD_SERVICE_OUT).to_dict()])
-            rc = SBus.send(pipe_path, dtg)
-            if (rc < 0):
-                return -1
-
-            reply = os.read(read_fd, 10)
-
-        res, error_txt = self._parse_sandbox_factory_answer(reply)
-        if res is True:
-            return 1
-        self.logger.error('Failed to ping to daemon factory: %s' % error_txt)
-        return 0
+        pipe_path = self.paths.host_factory_pipe
+        client = SBusClient(pipe_path)
+        try:
+            resp = client.ping()
+            if resp.status:
+                return 1
+            else:
+                self.logger.error('Failed to ping to daemon factory: %s' %
+                                  resp.message)
+                return 0
+        except SBusClientException:
+            return -1
 
     def wait(self):
         """
@@ -314,7 +281,7 @@ class RunTimeSandbox(object):
         Restarts the scope's sandbox using the specified docker image
 
         :param docker_image_name: name of the docker image to start
-        :returns: returned value of restart_docker_container
+        :raises StorletRuntimeException: when failed to restart the container
         """
         if self.docker_repo:
             docker_image_name = '%s/%s' % (self.docker_repo,
@@ -323,13 +290,10 @@ class RunTimeSandbox(object):
         docker_container_name = '%s_%s' % (self.docker_image_name_prefix,
                                            self.scope)
 
-        pipe_mount = '%s:%s' % (self.paths.host_pipe_prefix(),
-                                self.paths.sandbox_pipe_prefix)
-        storlet_mount = '%s:%s:ro' % (self.paths.host_storlet_prefix(),
-                                      self.paths.sandbox_storlet_dir_prefix)
-        storlet_python_lib_mount = '%s:%s:ro' % (
-            self.paths.host_storlet_python_lib_dir,
-            self.paths.sandbox_storlet_python_lib_dir)
+        pipe_mount = '%s:%s' % (self.paths.host_pipe_dir,
+                                self.paths.sandbox_pipe_dir)
+        storlet_mount = '%s:%s:ro' % (self.paths.host_storlet_base_dir,
+                                      self.paths.sandbox_storlet_base_dir)
         storlet_native_lib_mount = '%s:%s:ro' % (
             self.paths.host_storlet_native_lib_dir,
             self.paths.sandbox_storlet_native_lib_dir)
@@ -340,123 +304,106 @@ class RunTimeSandbox(object):
         cmd = [os.path.join(self.paths.host_restart_script_dir,
                             'restart_docker_container'),
                docker_container_name, docker_image_name, pipe_mount,
-               storlet_mount, storlet_python_lib_mount,
-               storlet_native_lib_mount, storlet_native_bin_mount]
+               storlet_mount, storlet_native_lib_mount,
+               storlet_native_bin_mount]
 
-        self.logger.debug('About to start container %s' % cmd)
+        proc = subprocess.Popen(cmd, stdout=subprocess.PIPE,
+                                stderr=subprocess.PIPE)
+        stdout, stderr = proc.communicate()
 
-        return subprocess.call(cmd)
+        if stdout:
+            if not isinstance(stdout, str):
+                stdout = stdout.decode("utf-8")
+            self.logger.debug('STDOUT: %s' % stdout.replace('\n', '#012'))
+        if stderr:
+            if not isinstance(stderr, str):
+                stderr = stderr.decode("utf-8")
+            self.logger.error('STDERR: %s' % stderr.replace('\n', '#012'))
+
+        if proc.returncode:
+            raise StorletRuntimeException('Failed to restart docker container')
 
     def restart(self):
         """
         Restarts the scope's sandbox
 
         """
-        self.paths.create_host_pipe_prefix()
+        self.paths.create_host_pipe_dir()
 
         docker_image_name = self.scope
-        if self._restart(docker_image_name) == 0:
+        try:
+            self._restart(docker_image_name)
             self.wait()
-            return
+        except StorletTimeout:
+            raise
+        except StorletRuntimeException:
+            # We were unable to start docker container from the tenant image.
+            # Let us try to start docker container from default image.
+            self.logger.exception("Failed to start docker container from "
+                                  "tenant image %s" % docker_image_name)
 
-        # We were unable to start docker container from the tenant image.
-        # Let us try to start docker container from default image.
-        self.logger.info("Failed to start docker container from tenant image "
-                         "%s" % docker_image_name)
-        self.logger.info("Trying to start docker container from default image")
-
-        # TODO(eranr): move the default tenant image name to a config var
-        docker_image_name = 'ubuntu_16.04_jre8_storlets'
-        self._restart(docker_image_name)
-        self.wait()
+            self.logger.info("Trying to start docker container from default "
+                             "image: %s" % self.default_docker_image_name)
+            self._restart(self.default_docker_image_name)
+            self.wait()
 
     def start_storlet_daemon(
             self, spath, storlet_id, language, language_version=None):
         """
         Start SDaemon process in the scope's sandbox
-
         """
-        prms = {'daemon_language': language.lower(),
-                'storlet_path': spath,
-                'storlet_name': storlet_id,
-                'uds_path': self.paths.sbox_storlet_pipe(storlet_id),
-                'log_level': self.storlet_daemon_debug_level,
-                'pool_size': self.storlet_daemon_thread_pool_size}
+        pipe_path = self.paths.host_factory_pipe
+        client = SBusClient(pipe_path)
+        try:
+            resp = client.start_daemon(
+                language.lower(), spath, storlet_id,
+                self.paths.get_sbox_storlet_pipe(storlet_id),
+                self.storlet_daemon_debug_level,
+                self.storlet_daemon_thread_pool_size,
+                language_version)
 
-        if language_version:
-            prms.update({'daemon_language_version': language_version})
-
-        with _open_pipe() as (read_fd, write_fd):
-            dtg = SBusServiceDatagram(
-                sbus_cmd.SBUS_CMD_START_DAEMON,
-                [write_fd],
-                [FDMetadata(sbus_fd.SBUS_FD_SERVICE_OUT).to_dict()],
-                prms)
-
-            pipe_path = self.paths.host_factory_pipe()
-            rc = SBus.send(pipe_path, dtg)
-            # TODO(takashi): Why we should rond rc into -1?
-            if (rc < 0):
-                return -1
-
-            reply = os.read(read_fd, 10)
-
-        res, error_txt = self._parse_sandbox_factory_answer(reply)
-        if res is True:
-            return 1
-        self.logger.error('Failed to start storlet daemon: %s' % error_txt)
-        return 0
+            if resp.status:
+                return 1
+            else:
+                self.logger.error('Failed to start storlet daemon: %s' %
+                                  resp.message)
+                return 0
+        except SBusClientException:
+            return -1
 
     def stop_storlet_daemon(self, storlet_id):
         """
         Stop SDaemon process in the scope's sandbox
         """
-        with _open_pipe() as (read_fd, write_fd):
-            dtg = SBusServiceDatagram(
-                sbus_cmd.SBUS_CMD_STOP_DAEMON,
-                [write_fd],
-                [FDMetadata(sbus_fd.SBUS_FD_SERVICE_OUT).to_dict()],
-                {'storlet_name': storlet_id})
-            pipe_path = self.paths.host_factory_pipe()
-            rc = SBus.send(pipe_path, dtg)
-            if (rc < 0):
-                self.logger.info("Failed to send status command to %s %s" %
-                                 (self.scope, storlet_id))
-                return -1
-
-            reply = os.read(read_fd, 10)
-
-        res, error_txt = self._parse_sandbox_factory_answer(reply)
-        if res is True:
-            return 1
-        self.logger.error('Failed to stop storlet daemon: %s' % error_txt)
-        return 0
+        pipe_path = self.paths.host_factory_pipe
+        client = SBusClient(pipe_path)
+        try:
+            resp = client.stop_daemon(storlet_id)
+            if resp.status:
+                return 1
+            else:
+                self.logger.error('Failed to stop storlet daemon: %s' %
+                                  resp.message)
+                return 0
+        except SBusClientException:
+            return -1
 
     def get_storlet_daemon_status(self, storlet_id):
         """
         Get the status of SDaemon process in the scope's sandbox
         """
-        with _open_pipe() as (read_fd, write_fd):
-            dtg = SBusServiceDatagram(
-                sbus_cmd.SBUS_CMD_DAEMON_STATUS,
-                [write_fd],
-                [FDMetadata(sbus_fd.SBUS_FD_SERVICE_OUT).to_dict()],
-                {'storlet_name': storlet_id})
-            pipe_path = self.paths.host_factory_pipe()
-            rc = SBus.send(pipe_path, dtg)
-            if (rc < 0):
-                self.logger.info("Failed to send status command to %s %s" %
-                                 (self.scope, storlet_id))
-                return -1
-
-            reply = os.read(read_fd, 10)
-
-        res, error_txt = self._parse_sandbox_factory_answer(reply)
-        if res is True:
-            return 1
-        self.logger.error('Failed to get status about storlet daemon: %s' %
-                          error_txt)
-        return 0
+        pipe_path = self.paths.host_factory_pipe
+        client = SBusClient(pipe_path)
+        try:
+            resp = client.daemon_status(storlet_id)
+            if resp.status:
+                return 1
+            else:
+                self.logger.error('Failed to get status about storlet '
+                                  'daemon: %s' % resp.message)
+                return 0
+        except SBusClientException:
+            return -1
 
     def _get_storlet_classpath(self, storlet_main, storlet_id, dependencies):
         """
@@ -467,12 +414,11 @@ class RunTimeSandbox(object):
         :param dependencies: A list of dependency file
         :returns: classpath string
         """
-        class_path = os.path.join(self.paths.sbox_storlet_exec(storlet_main),
-                                  storlet_id)
+        class_path = os.path.join(
+            self.paths.get_sbox_storlet_dir(storlet_main), storlet_id)
 
         dep_path_list = \
-            [os.path.join(self.paths.sbox_storlet_exec(storlet_main),
-                          dep)
+            [os.path.join(self.paths.get_sbox_storlet_dir(storlet_main), dep)
              for dep in dependencies]
 
         return class_path + ':' + ':'.join(dep_path_list)
@@ -485,11 +431,7 @@ class RunTimeSandbox(object):
             # Best we can do is execute the container.
             self.logger.debug('Failed to check Storlet daemon status, '
                               'restart Docker container')
-            try:
-                self.restart()
-            except StorletTimeout:
-                raise StorletRuntimeException('Docker container is '
-                                              'not responsive')
+            self.restart()
             storlet_daemon_status = 0
 
         if (cache_updated is True and storlet_daemon_status == 1):
@@ -499,11 +441,7 @@ class RunTimeSandbox(object):
                               'is running. Stopping daemon')
             res = self.stop_storlet_daemon(sreq.storlet_main)
             if res != 1:
-                try:
-                    self.restart()
-                except StorletTimeout:
-                    raise StorletRuntimeException('Docker container is '
-                                                  'not responsive')
+                self.restart()
             else:
                 self.logger.debug('Deamon stopped')
             storlet_daemon_status = 0
@@ -517,7 +455,7 @@ class RunTimeSandbox(object):
 
             daemon_status = self.start_storlet_daemon(
                 classpath, sreq.storlet_main, sreq.storlet_language,
-                sreq.options.get("storlet_language_version"))
+                sreq.storlet_language_version)
 
             if daemon_status != 1:
                 self.logger.error('Daemon start Failed, returned code is %d' %
@@ -526,6 +464,7 @@ class RunTimeSandbox(object):
             else:
                 self.logger.debug('Daemon started')
 
+
 """---------------------------------------------------------------------------
 Storlet Daemon API
 StorletInvocationProtocol
@@ -555,9 +494,7 @@ class StorletInvocationProtocol(object):
                  timeout, logger, extra_sources=None):
         self.srequest = srequest
         self.storlet_pipe_path = storlet_pipe_path
-        self.storlet_logger_path = storlet_logger_path
-        self.storlet_logger = StorletLogger(self.storlet_logger_path,
-                                            'storlet_invoke')
+        self.storlet_logger = StorletLogger(storlet_logger_path)
         self.logger = logger
         self.timeout = timeout
 
@@ -566,8 +503,6 @@ class StorletInvocationProtocol(object):
         self.data_write_fd = None
         self.metadata_read_fd = None
         self.metadata_write_fd = None
-        self.execution_str_read_fd = None
-        self.execution_str_write_fd = None
         self.task_id = None
         self._input_data_read_fd = None
         self._input_data_write_fd = None
@@ -584,9 +519,6 @@ class StorletInvocationProtocol(object):
                  'user_metadata': source.user_metadata,
                  'data_iter': source.data_iter})
 
-        if not os.path.exists(storlet_logger_path):
-            os.makedirs(storlet_logger_path)
-
     @property
     def input_data_read_fd(self):
         """
@@ -600,47 +532,33 @@ class StorletInvocationProtocol(object):
     @property
     def remote_fds(self):
         """
-        File descriptors to be passed to container side
+        A list of sbus file descirptors passed to remote side
         """
-        remote_fds = [self.input_data_read_fd,
-                      self.execution_str_write_fd,
-                      self.data_write_fd,
-                      self.metadata_write_fd,
-                      self.storlet_logger.getfd()]
-
-        for source in self.extra_data_sources:
-            remote_fds.append(source['read_fd'])
-        return remote_fds
-
-    @property
-    def remote_fds_metadata(self):
-        """
-        Metadata about file descriptors to be passed to container side
-        """
-        input_fd_metadata = FDMetadata(
-            sbus_fd.SBUS_FD_INPUT_OBJECT,
-            storage_metadata=self.srequest.user_metadata)
-        if self.srequest.user_metadata:
-            input_fd_metadata.storage_metadata.update(
-                self.srequest.user_metadata)
+        storlets_metadata = {}
         if self.srequest.has_range:
-            input_fd_metadata.storlets_metadata['start'] = \
-                str(self.srequest.start)
-            input_fd_metadata.storlets_metadata['end'] = \
-                str(self.srequest.end)
-        fds_metadata = [
-            input_fd_metadata.to_dict(),
-            FDMetadata(sbus_fd.SBUS_FD_OUTPUT_TASK_ID).to_dict(),
-            FDMetadata(sbus_fd.SBUS_FD_OUTPUT_OBJECT).to_dict(),
-            FDMetadata(sbus_fd.SBUS_FD_OUTPUT_OBJECT_METADATA).to_dict(),
-            FDMetadata(sbus_fd.SBUS_FD_LOGGER).to_dict()]
+            storlets_metadata.update(
+                {'start': str(self.srequest.start),
+                 'end': str(self.srequest.end)})
+
+        fds = [SBusFileDescriptor(sbus_fd.SBUS_FD_INPUT_OBJECT,
+                                  self.input_data_read_fd,
+                                  storage_metadata=self.srequest.user_metadata,
+                                  storlets_metadata=storlets_metadata),
+               SBusFileDescriptor(sbus_fd.SBUS_FD_OUTPUT_OBJECT,
+                                  self.data_write_fd),
+               SBusFileDescriptor(sbus_fd.SBUS_FD_OUTPUT_OBJECT_METADATA,
+                                  self.metadata_write_fd),
+               SBusFileDescriptor(sbus_fd.SBUS_FD_LOGGER,
+                                  self.storlet_logger.getfd())]
 
         for source in self.extra_data_sources:
-            fdmd = FDMetadata(
+            fd = SBusFileDescriptor(
                 sbus_fd.SBUS_FD_INPUT_OBJECT,
+                source['read_fd'],
                 storage_metadata=source['user_metadata'])
-            fds_metadata.append(fdmd.to_dict())
-        return fds_metadata
+            fds.append(fd)
+
+        return fds
 
     @contextmanager
     def _activate_invocation_descriptors(self):
@@ -663,7 +581,6 @@ class StorletInvocationProtocol(object):
         if not self.srequest.has_fd:
             self._input_data_read_fd, self._input_data_write_fd = os.pipe()
         self.data_read_fd, self.data_write_fd = os.pipe()
-        self.execution_str_read_fd, self.execution_str_write_fd = os.pipe()
         self.metadata_read_fd, self.metadata_write_fd = os.pipe()
 
         for source in self.extra_data_sources:
@@ -690,8 +607,7 @@ class StorletInvocationProtocol(object):
         """
         Close all of the container side descriptors
         """
-        fds = [self.data_write_fd, self.metadata_write_fd,
-               self.execution_str_write_fd]
+        fds = [self.data_write_fd, self.metadata_write_fd]
         if not self.srequest.has_fd:
             fds.append(self.input_data_read_fd)
         fds.extend([source['read_fd'] for source in self.extra_data_sources])
@@ -702,8 +618,7 @@ class StorletInvocationProtocol(object):
         """
         Close all of the host side descriptors
         """
-        fds = [self.data_read_fd, self.metadata_read_fd,
-               self.execution_str_read_fd]
+        fds = [self.data_read_fd, self.metadata_read_fd]
         fds.extend([source['write_fd'] for source in self.extra_data_sources])
         self._safe_close(fds)
 
@@ -711,18 +626,13 @@ class StorletInvocationProtocol(object):
         """
         Cancel on-going storlet execution
         """
-        with _open_pipe() as (read_fd, write_fd):
-            dtg = SBusServiceDatagram(
-                sbus_cmd.SBUS_CMD_CANCEL,
-                [write_fd],
-                [FDMetadata(sbus_fd.SBUS_FD_SERVICE_OUT).to_dict()],
-                None,
-                self.task_id)
-            rc = SBus.send(self.storlet_pipe_path, dtg)
-            if (rc < 0):
+        client = SBusClient(self.storlet_pipe_path)
+        try:
+            resp = client.cancel(self.task_id)
+            if not resp.status:
                 raise StorletRuntimeException('Failed to cancel task')
-            # TODO(takashi): Check the response here
-            os.read(read_fd, 10)
+        except SBusClientException:
+            raise StorletRuntimeException('Failed to cancel task')
 
     def _invoke(self):
         """
@@ -731,24 +641,23 @@ class StorletInvocationProtocol(object):
         with self.storlet_logger.activate(),\
                 self._activate_invocation_descriptors():
             self._send_execute_command()
-        self._wait_for_read_with_timeout(self.execution_str_read_fd)
-        # TODO(kota_): need an assertion for task_id format
-        self.task_id = os.read(self.execution_str_read_fd, 10)
-        os.close(self.execution_str_read_fd)
 
     def _send_execute_command(self):
         """
         Send execute command to the remote daemon factory to invoke storlet
         execution
         """
-        dtg = SBusExecuteDatagram(
-            sbus_cmd.SBUS_CMD_EXECUTE,
-            self.remote_fds,
-            self.remote_fds_metadata,
-            self.srequest.params)
-        rc = SBus.send(self.storlet_pipe_path, dtg)
+        client = SBusClient(self.storlet_pipe_path)
+        try:
+            resp = client.execute(self.srequest.params, self.remote_fds)
+            if not resp.status:
+                raise StorletRuntimeException("Failed to send execute command")
 
-        if (rc < 0):
+            if not resp.task_id:
+                raise StorletRuntimeException("Missing task id")
+            else:
+                self.task_id = resp.task_id
+        except SBusClientException:
             raise StorletRuntimeException("Failed to send execute command")
 
     def _wait_for_read_with_timeout(self, fd):
@@ -856,7 +765,7 @@ class StorletInvocationProtocol(object):
 
     @contextmanager
     def _open_writer(self, fd):
-        with os.fdopen(fd, 'w') as writer:
+        with os.fdopen(fd, 'wb') as writer:
             yield writer
 
     def _write_input_data(self, fd, data_iter):
