@@ -22,6 +22,9 @@ from contextlib import contextmanager
 from six import StringIO
 from stat import ST_MODE
 
+from storlets.sbus.client import SBusResponse
+from storlets.sbus.client.exceptions import SBusClientIOError, \
+    SBusClientMalformedResponse, SBusClientSendError
 from storlets.gateway.common.exceptions import StorletRuntimeException, \
     StorletTimeout
 from storlets.gateway.gateways.docker.gateway import DockerStorletRequest
@@ -31,14 +34,6 @@ from tests.unit import FakeLogger, with_tempdir
 from tests.unit.gateway.gateways import FakeFileManager
 
 
-@contextmanager
-def _mock_sbus(send_status=0):
-    with mock.patch('storlets.gateway.gateways.docker.runtime.'
-                    'SBus.send') as fake_send:
-        fake_send.return_value = send_status
-        yield
-
-
 @contextmanager
 def _mock_os_pipe(bufs):
     class FakeFd(object):
@@ -103,19 +98,20 @@ class TestRuntimePaths(unittest.TestCase):
     def tearDown(self):
         pass
 
-    def test_host_pipe_prefix(self):
+    def test_host_pipe_dir(self):
         self.assertEqual(
-            self.paths.host_pipe_prefix(),
-            os.path.join(self.pipes_dir, self.scope))
+            os.path.join(self.pipes_dir, self.scope),
+            self.paths.host_pipe_dir)
 
-    def test_create_host_pipe_prefix(self):
-        pipedir = self.paths.host_pipe_prefix()
+    def test_create_host_pipe_dir(self):
+        pipedir = self.paths.host_pipe_dir
 
         # When the directory exists
         with mock.patch('os.path.exists', return_value=True), \
                 mock.patch('os.makedirs') as m, \
                 mock.patch('os.chmod') as c:
-            self.paths.create_host_pipe_prefix()
+            self.assertEqual(os.path.join(self.pipes_dir, self.scope),
+                             self.paths.create_host_pipe_dir())
             self.assertEqual(0, m.call_count)
             cargs, ckwargs = c.call_args
             # Make sure about the target directory
@@ -125,7 +121,8 @@ class TestRuntimePaths(unittest.TestCase):
         with mock.patch('os.path.exists', return_value=False), \
                 mock.patch('os.makedirs') as m, \
                 mock.patch('os.chmod') as c:
-            self.paths.create_host_pipe_prefix(),
+            self.assertEqual(os.path.join(self.pipes_dir, self.scope),
+                             self.paths.create_host_pipe_dir())
             self.assertEqual(1, m.call_count)
             # Make sure about the target directory
             margs, mkwargs = m.call_args
@@ -135,61 +132,49 @@ class TestRuntimePaths(unittest.TestCase):
 
     def test_host_factory_pipe(self):
         self.assertEqual(
-            self.paths.host_factory_pipe(),
+            self.paths.host_factory_pipe,
             os.path.join(self.pipes_dir, self.scope, 'factory_pipe'))
 
-    def test_host_storlet_pipe(self):
+    def test_get_host_storlet_pipe(self):
         self.assertEqual(
-            self.paths.host_storlet_pipe(self.storlet_id),
-            os.path.join(self.pipes_dir, self.scope, self.storlet_id))
+            os.path.join(self.pipes_dir, self.scope, self.storlet_id),
+            self.paths.get_host_storlet_pipe(self.storlet_id))
 
-    def test_sbox_storlet_pipe(self):
+    def test_get_sbox_storlet_pipe(self):
         self.assertEqual(
-            self.paths.sbox_storlet_pipe(self.storlet_id),
-            os.path.join('/mnt/channels', self.storlet_id))
+            os.path.join('/mnt/channels', self.storlet_id),
+            self.paths.get_sbox_storlet_pipe(self.storlet_id))
 
-    def test_sbox_storlet_exec(self):
+    def test_get_sbox_storlet_dir(self):
         self.assertEqual(
-            self.paths.sbox_storlet_exec(self.storlet_id),
-            os.path.join('/home/swift', self.storlet_id))
+            os.path.join('/home/swift', self.storlet_id),
+            self.paths.get_sbox_storlet_dir(self.storlet_id))
 
-    def test_host_storlet_prefix(self):
+    def test_host_storlet_base_dir(self):
         self.assertEqual(
-            self.paths.host_storlet_prefix(),
+            self.paths.host_storlet_base_dir,
             os.path.join(self.storlets_dir, self.scope))
 
-    def test_host_storlet(self):
+    def test_get_host_storlet_dir(self):
         self.assertEqual(
-            self.paths.host_storlet(self.storlet_id),
-            os.path.join(self.storlets_dir, self.scope,
-                         self.storlet_id))
+            os.path.join(self.storlets_dir, self.scope, self.storlet_id),
+            self.paths.get_host_storlet_dir(self.storlet_id))
 
-    def test_slog_path(self):
-        with mock.patch('os.path.exists', return_value=True), \
-            mock.patch('os.makedirs') as m:
-            self.assertEqual(
-                self.paths.slog_path(self.storlet_id),
-                os.path.join(self.log_dir, self.scope,
-                             self.storlet_id))
-            self.assertEqual(0, m.call_count)
-
-        with mock.patch('os.path.exists', return_value=False), \
-            mock.patch('os.makedirs') as m:
-            self.assertEqual(
-                self.paths.slog_path(self.storlet_id),
-                os.path.join(self.log_dir, self.scope,
-                             self.storlet_id))
-            self.assertEqual(1, m.call_count)
-
-    def test_get_host_storlet_cache_dir(self):
+    def test_get_host_slog_path(self):
         self.assertEqual(
-            self.paths.get_host_storlet_cache_dir(),
-            os.path.join(self.cache_dir, self.scope, 'storlet'))
+            os.path.join(self.log_dir, self.scope, self.storlet_id,
+                         'storlet_invoke.log'),
+            self.paths.get_host_slog_path(self.storlet_id))
 
-    def test_get_host_dependency_cache_dir(self):
+    def test_host_storlet_cache_dir(self):
         self.assertEqual(
-            self.paths.get_host_dependency_cache_dir(),
-            os.path.join(self.cache_dir, self.scope, 'dependency'))
+            os.path.join(self.cache_dir, self.scope, 'storlet'),
+            self.paths.host_storlet_cache_dir)
+
+    def test_host_dependency_cache_dir(self):
+        self.assertEqual(
+            os.path.join(self.cache_dir, self.scope, 'dependency'),
+            self.paths.host_dependency_cache_dir)
 
     def test_runtime_paths_default(self):
         # CHECK: docs  says we need 4 dirs for communicate
@@ -212,36 +197,36 @@ class TestRuntimePaths(unittest.TestCase):
 
         # For pipe
         self.assertEqual('/home/docker_device/pipes/scopes/account',
-                         runtime_paths.host_pipe_prefix())
+                         runtime_paths.host_pipe_dir)
 
         # 1. host_factory_pipe_path <pipes_dir>/<scope>/factory_pipe
         self.assertEqual(
             '/home/docker_device/pipes/scopes/account/factory_pipe',
-            runtime_paths.host_factory_pipe())
+            runtime_paths.host_factory_pipe)
         # 2. host_storlet_pipe_path <pipes_dir>/<scope>/<storlet_id>
         self.assertEqual(
             '/home/docker_device/pipes/scopes/account/Storlet-1.0.jar',
-            runtime_paths.host_storlet_pipe(storlet_id))
+            runtime_paths.get_host_storlet_pipe(storlet_id))
         # 3. Yes, right now, we don't have the path for #3 in Python
         # 4. sandbox_storlet_pipe_path | /mnt/channels/<storlet_id>
         self.assertEqual('/mnt/channels/Storlet-1.0.jar',
-                         runtime_paths.sbox_storlet_pipe(storlet_id))
+                         runtime_paths.get_sbox_storlet_pipe(storlet_id))
 
         # This looks like for jar load?
         self.assertEqual('/home/docker_device/storlets/scopes/account',
-                         runtime_paths.host_storlet_prefix())
+                         runtime_paths.host_storlet_base_dir)
         self.assertEqual(
             '/home/docker_device/storlets/scopes/account/Storlet-1.0.jar',
-            runtime_paths.host_storlet(storlet_id))
+            runtime_paths.get_host_storlet_dir(storlet_id))
         # And this one is a mount poit in sand box?
         self.assertEqual('/home/swift/Storlet-1.0.jar',
-                         runtime_paths.sbox_storlet_exec(storlet_id))
+                         runtime_paths.get_sbox_storlet_dir(storlet_id))
 
     @with_tempdir
-    def test_create_host_pipe_prefix_with_real_dir(self, temp_dir):
+    def test_create_host_pipe_dir_with_real_dir(self, temp_dir):
         runtime_paths = RunTimePaths('account', {'host_root': temp_dir})
-        runtime_paths.create_host_pipe_prefix()
-        path = runtime_paths.host_pipe_prefix()
+        runtime_paths.create_host_pipe_dir()
+        path = runtime_paths.host_pipe_dir
         self.assertTrue(os.path.exists(path))
         self.assertTrue(os.path.isdir(path))
         permission = oct(os.stat(path)[ST_MODE])[-3:]
@@ -263,89 +248,130 @@ class TestRunTimeSandbox(unittest.TestCase):
         self.scope = '0123456789abc'
         self.sbox = RunTimeSandbox(self.scope, self.conf, self.logger)
 
-    def tearDown(self):
-        pass
-
-    def test_parse_sandbox_factory_answer(self):
-        status, msg = self.sbox._parse_sandbox_factory_answer('True:message')
-        self.assertTrue(status)
-        self.assertEqual('message', msg)
-
-        status, msg = self.sbox._parse_sandbox_factory_answer('False:message')
-        self.assertFalse(status)
-        self.assertEqual('message', msg)
-
-        with self.assertRaises(StorletRuntimeException):
-            self.sbox._parse_sandbox_factory_answer('Foo')
-
-    def _check_all_pipese_closed(self, pipes):
-        for _pipe in pipes:
-            self.assertTrue(_pipe[0].closed)
-            self.assertTrue(_pipe[1].closed)
-
     def test_ping(self):
-        with _mock_os_pipe(['True:OK']) as pipes, _mock_sbus(0):
-            self.assertEqual(1, self.sbox.ping())
-            self._check_all_pipese_closed(pipes)
+        with mock.patch('storlets.gateway.gateways.docker.runtime.'
+                        'SBusClient.ping') as ping:
+            ping.return_value = SBusResponse(True, 'OK')
+            self.assertEqual(self.sbox.ping(), 1)
 
-        with _mock_os_pipe(['False:ERROR']) as pipes, _mock_sbus(-1):
-            self.assertEqual(-1, self.sbox.ping())
-            self._check_all_pipese_closed(pipes)
+        with mock.patch('storlets.gateway.gateways.docker.runtime.'
+                        'SBusClient.ping') as ping:
+            ping.return_value = SBusResponse(False, 'Error')
+            self.assertEqual(self.sbox.ping(), 0)
 
-        with _mock_os_pipe(['Foo']) as pipes, _mock_sbus(0):
-            with self.assertRaises(StorletRuntimeException):
-                self.sbox.ping()
-            self._check_all_pipese_closed(pipes)
+        with mock.patch('storlets.gateway.gateways.docker.runtime.'
+                        'SBusClient.ping') as ping:
+            ping.side_effect = SBusClientSendError()
+            self.assertEqual(self.sbox.ping(), -1)
+
+        with mock.patch('storlets.gateway.gateways.docker.runtime.'
+                        'SBusClient.ping') as ping:
+            ping.side_effect = SBusClientMalformedResponse()
+            self.assertEqual(self.sbox.ping(), -1)
+
+        with mock.patch('storlets.gateway.gateways.docker.runtime.'
+                        'SBusClient.ping') as ping:
+            ping.side_effect = SBusClientIOError()
+            self.assertEqual(self.sbox.ping(), -1)
 
     def test_wait(self):
-        with _mock_os_pipe(['True:OK']) as pipes, _mock_sbus(0), \
+        with mock.patch('storlets.gateway.gateways.docker.runtime.'
+                        'SBusClient.ping') as ping, \
             mock.patch('storlets.gateway.gateways.docker.runtime.'
-                       'time.sleep') as _s:
+                       'time.sleep') as sleep:
+            ping.return_value = SBusResponse(True, 'OK')
             self.sbox.wait()
-            self.assertEqual(0, _s.call_count)
-            self._check_all_pipese_closed(pipes)
+            self.assertEqual(sleep.call_count, 0)
 
-        with _mock_os_pipe(['False:ERROR', 'True:OK']) as pipes, \
-            _mock_sbus(0), \
+        with mock.patch('storlets.gateway.gateways.docker.runtime.'
+                        'SBusClient.ping') as ping, \
             mock.patch('storlets.gateway.gateways.docker.runtime.'
-                       'time.sleep') as _s:
+                       'time.sleep') as sleep:
+            ping.side_effect = [SBusResponse(False, 'Error'),
+                                SBusResponse(True, 'OK')]
             self.sbox.wait()
-            self.assertEqual(1, _s.call_count)
-            self._check_all_pipese_closed(pipes)
-
-        with _mock_os_pipe(['Foo']) as pipes, _mock_sbus(0), \
-            mock.patch('storlets.gateway.gateways.docker.runtime.'
-                       'time.sleep') as _s:
-            with self.assertRaises(StorletRuntimeException):
-                self.sbox.wait()
-            self._check_all_pipese_closed(pipes)
+            self.assertEqual(sleep.call_count, 1)
 
         # TODO(takashi): should test timeout case
 
     def test_restart(self):
+
+        class FakeProc(object):
+            def __init__(self, stdout, stderr, code):
+                self.stdout = stdout
+                self.stderr = stderr
+                self.returncode = code
+
+            def communicate(self):
+                return (self.stdout, self.stderr)
+
         with mock.patch('storlets.gateway.gateways.docker.runtime.'
-                        'RunTimePaths.create_host_pipe_prefix'), \
+                        'RunTimePaths.create_host_pipe_dir'), \
             mock.patch('storlets.gateway.gateways.docker.runtime.'
-                       'subprocess.call'):
+                       'subprocess.Popen') as popen:
             _wait = self.sbox.wait
 
             def dummy_wait_success(*args, **kwargs):
                 return 1
 
             self.sbox.wait = dummy_wait_success
+
+            # Test that popen is called successfully
+            popen.return_value = FakeProc('Try to restart\nOK', '', 0)
             self.sbox.restart()
+            self.assertEqual(1, popen.call_count)
             self.sbox.wait = _wait
 
         with mock.patch('storlets.gateway.gateways.docker.runtime.'
-                        'RunTimePaths.create_host_pipe_prefix'), \
+                        'RunTimePaths.create_host_pipe_dir'), \
             mock.patch('storlets.gateway.gateways.docker.runtime.'
-                       'subprocess.call'):
+                       'subprocess.Popen') as popen:
+            _wait = self.sbox.wait
+
+            def dummy_wait_success(*args, **kwargs):
+                return 1
+
+            self.sbox.wait = dummy_wait_success
+
+            # Test double failure to restart the container
+            # for both the tenant image and generic image
+            popen.return_value = FakeProc('Try to restart', 'Some error', 1)
+            with self.assertRaises(StorletRuntimeException):
+                self.sbox.restart()
+            self.assertEqual(2, popen.call_count)
+            self.sbox.wait = _wait
+
+        with mock.patch('storlets.gateway.gateways.docker.runtime.'
+                        'RunTimePaths.create_host_pipe_dir'), \
+            mock.patch('storlets.gateway.gateways.docker.runtime.'
+                       'subprocess.Popen') as popen:
+            _wait = self.sbox.wait
+
+            def dummy_wait_success(*args, **kwargs):
+                return 1
+
+            self.sbox.wait = dummy_wait_success
+
+            # Test failure to restart the container for the tenant image
+            # success for the generic image
+            popen.side_effect = [FakeProc('Try to restart', 'Some error', 1),
+                                 FakeProc('Try to restart\nOK', '', 0)]
+            self.sbox.restart()
+            self.assertEqual(2, popen.call_count)
+            self.sbox.wait = _wait
+
+        with mock.patch('storlets.gateway.gateways.docker.runtime.'
+                        'RunTimePaths.create_host_pipe_dir'), \
+            mock.patch('storlets.gateway.gateways.docker.runtime.'
+                       'subprocess.Popen') as popen:
             _wait = self.sbox.wait
 
             def dummy_wait_failure(*args, **kwargs):
-                raise StorletRuntimeException()
+                raise StorletTimeout()
 
             self.sbox.wait = dummy_wait_failure
+
+            popen.return_value = FakeProc('OK', '', 0)
             with self.assertRaises(StorletRuntimeException):
                 self.sbox.restart()
             self.sbox.wait = _wait
@@ -384,13 +410,39 @@ class TestStorletInvocationProtocol(unittest.TestCase):
             except OSError:
                 pass
 
-    def test_invocation_protocol(self):
-        # os.pipe will be called 4 times
-        pipe_called = 4
+    def test_send_execute_command(self):
+        with mock.patch('storlets.gateway.gateways.docker.runtime.SBusClient.'
+                        'execute') as execute:
+            execute.return_value = SBusResponse(True, 'OK', 'someid')
+            self.protocol._send_execute_command()
+            self.assertEqual('someid', self.protocol.task_id)
 
-        with _mock_sbus(0), _mock_os_pipe([''] * pipe_called) as pipes:
-            with mock.patch.object(
-                    self.protocol, '_wait_for_read_with_timeout'):
+        with mock.patch('storlets.gateway.gateways.docker.runtime.SBusClient.'
+                        'execute') as execute:
+            execute.return_value = SBusResponse(True, 'OK')
+            with self.assertRaises(StorletRuntimeException):
+                self.protocol._send_execute_command()
+
+        with mock.patch('storlets.gateway.gateways.docker.runtime.SBusClient.'
+                        'execute') as execute:
+            execute.return_value = SBusResponse(False, 'NG', 'someid')
+            with self.assertRaises(StorletRuntimeException):
+                self.protocol._send_execute_command()
+
+        with mock.patch('storlets.gateway.gateways.docker.runtime.SBusClient.'
+                        'execute') as execute:
+            execute.side_effect = SBusClientIOError()
+            with self.assertRaises(StorletRuntimeException):
+                self.protocol._send_execute_command()
+
+    def test_invocation_protocol(self):
+        # os.pipe will be called 3 times
+        pipe_called = 3
+
+        with _mock_os_pipe([''] * pipe_called) as pipes:
+            with mock.patch.object(self.protocol,
+                                   '_wait_for_read_with_timeout'), \
+                    mock.patch.object(self.protocol, '_send_execute_command'):
                 self.protocol._invoke()
 
             self.assertEqual(pipe_called, len(pipes))
@@ -407,11 +459,6 @@ class TestStorletInvocationProtocol(unittest.TestCase):
             self.assertFalse(data_read_fd.closed)
             self.assertTrue(data_write_fd.closed)
 
-            # both execution str fds are closed
-            execution_read_fd, execution_write_fd = next(pipes)
-            self.assertTrue(execution_read_fd.closed)
-            self.assertTrue(execution_write_fd.closed)
-
             # metadata write fd is closed, metadata read fd is still open.
             metadata_read_fd, metadata_write_fd = next(pipes)
             self.assertFalse(metadata_read_fd.closed)
@@ -421,12 +468,12 @@ class TestStorletInvocationProtocol(unittest.TestCase):
             self.assertRaises(StopIteration, next, pipes)
 
     def test_invocation_protocol_remote_fds(self):
-        # In default, we have 5 fds in remote_fds
+        # In default, we have 4 fds in remote_fds
         storlet_request = DockerStorletRequest(
             self.storlet_id, {}, {}, iter(StringIO()), options=self.options)
         protocol = StorletInvocationProtocol(
             storlet_request, self.pipe_path, self.log_file, 1, self.logger)
-        self.assertEqual(5, len(protocol.remote_fds))
+        self.assertEqual(4, len(protocol.remote_fds))
 
         # extra_resources expands the remote_fds
         storlet_request = DockerStorletRequest(
@@ -434,7 +481,7 @@ class TestStorletInvocationProtocol(unittest.TestCase):
         protocol = StorletInvocationProtocol(
             storlet_request, self.pipe_path, self.log_file, 1, self.logger,
             extra_sources=[storlet_request])
-        self.assertEqual(6, len(protocol.remote_fds))
+        self.assertEqual(5, len(protocol.remote_fds))
 
         # 2 more extra_resources expands the remote_fds
         storlet_request = DockerStorletRequest(
@@ -442,7 +489,7 @@ class TestStorletInvocationProtocol(unittest.TestCase):
         protocol = StorletInvocationProtocol(
             storlet_request, self.pipe_path, self.log_file, 1, self.logger,
             extra_sources=[storlet_request] * 3)
-        self.assertEqual(8, len(protocol.remote_fds))
+        self.assertEqual(7, len(protocol.remote_fds))
 
     def test_open_writer_with_invalid_fd(self):
         invalid_fds = (
